/*
 * bucket.h
 *
 *  Created on: Jun 10, 2015
 *      Author: makoto
 */

#ifndef BUCKET_H_
#define BUCKET_H_



// Address on I2C
#define I2C_SLAVE_DEVICE_START_ADDR	0x20
#define I2C_SLAVE_DEVICE_END_ADDR	0x30
#define I2C_ADDR_LENS 0x20
#define I2C_ADDR_STROBE 0x62
#define I2C_ADDR_MCU 0x24
#define I2C_ADDR_LINEAR_ACTUATOR 0x33
#define I2C_ADDR_EXPANDER 0x20

#define MAX_NUM_SCAN_DOWNSTREAM 0x4
#define BUFFER_LENGTH 128

#define OFFSET_DATA_LENGTH_HEADER 1
#define OFFSET_FIRST_COMPONENT 2

#define OFFSET_START_CODE 1
#define OFFSET_ADDRESS_HIERARCHY 1
#define OFFSET_ADDRESS_HORIZONTAL 2

#define OFFSET_DOWNSTREAM_CODE 9

// COMMAND
#define I2C_COMMAND_START_OF_DATA 0xFB
#define I2C_COMMAND_END_OF_DATA 0xFC
#define I2C_COMMAND_OPENING_BRANCH 0xFD
#define I2C_COMMAND_CLOSING_BRANCH 0xFE

#define I2C_CLK 16

#define I2C_COMMAND_SERVO_POSITION 0x1
#define I2C_COMMAND_SERVO_TIME 0x2
#define I2C_COMMAND_SERVO_POSITION_DIRECT 0x3
#define I2C_COMMAND_LINEAR_ACTUATOR_MOVE_TARGET 0x4
#define I2C_COMMAND_LINEAR_ACTUATOR_SET_VELOCITY 0x5
#define I2C_COMMAND_LINEAR_ACTUATOR_INITIALIZE 0x6

#define I2C_COMMAND_FLASH_STROBE 0x8
#define I2C_COMMAND_FLASH_STROBE_PRESCALE 0x9
#define I2C_COMMAND_FLASH_STROBE_DUTY 0xA


#define I2C_COMMAND_MCU2PC_ENABLE 100
#define I2C_COMMAND_MCU2PC_DISABLE 101


// COMMAND FOR USB
#define USB_COMMAND_START_OF_DATA 0xFA

// MAJOR TYPE
#define TYPE_BASE 0
#define TYPE_SENSOR 1
#define TYPE_LENS 2
#define TYPE_ACTUATOR 3
#define TYPE_SPACER 4
#define TYPE_STROBE 5
#define TYPE_OPTICAL_ATTACHMENT 6
#define TYPE_AUDIO 7
#define TYPE_NO_DEFINE 0x7F
// SECOND Type
#define BASE_ZERO 0
#define BASE_1PORT 1
#define BASE_2PORT 2
#define BASE_3PORT 3
#define BASE_4PORT 4

#define SENSOR_1MP 0
#define SENSOR_3MP 1
#define SENSOR_8MP 2
#define SENSOR_1300KP 4

#define LENS_NORMAL_ANGLE 0
#define LENS_WIDE_ANGLE 1

#define LENS_FOCAL_STACK 5

//
#define LENGTH_COMPONENT_CODE 8
#define I2C_DETECT_ACK 1
#define I2C_DETECT_NACK 0
#define I2C_RETRY_MAX 1


#define MAXMUM_CONNECTION 8
#define I2C_IRQ_WAIT_TIME 500
#define I2C_IRQ_TIMEOUT 100
#define DOWNSTREAM_SCAN_RATIO 1

#define WRITE_PACKET_LENGTH 8
#define MAX_STR_LENGTH 64

#define DURATION_COMPONENT_MSEC 125

#define DURATION_COMPONENT_DELAY 4096

struct ComponentCodeAddr{
	unsigned char height;
	unsigned char breadth;
};
struct ComponentCodeType{
	unsigned char major;
	unsigned char second;
};
struct ComponentCodeSetting{
	unsigned char hi;
	unsigned char lo;
};

struct ComponentCode
{
	unsigned char start_code;
	struct ComponentCodeAddr addr;
	struct ComponentCodeType type;
	struct ComponentCodeSetting setting;
	unsigned char connection;

}stComponentCode;


struct I2C_WriteCommand
{
	unsigned char addr;
	unsigned char type;
	unsigned char command;
	unsigned char data[4];
	unsigned char reserved;
}stI2C_WriteCommand;



#include "component_type.h"


#endif /* BUCKET_H_ */
