/*
 * focal_stack.c
 *
 *  Created on: Sep 25, 2015
 *      Author: makoto
 */
#include "bucket.h"
#include "i2c_com.h"
#include "i2c_test.h"
#include "dec2hex.h"
#include "focal_stack.h"




extern unsigned char RxBuffer[BUFFER_LENGTH];

volatile int linear_actuator_flag;




void linear_actuator_debug_via_com()
{
	unsigned char data[4];
	volatile int velocity,position,current_pos;

	data[0]=0x30;
	data[1]=0x0;
	data[2]=0xE8;
	data[3]=0x3;

	velocity=(data[3]<<8)+data[2];
	position=(data[1]<<8)+data[0];

   	linear_actuator_calibrate_motor();				// <87 4> with wait for over 30msec
    linear_actuator_set_speed((data[3]<<8)+data[2],10,51);
    linear_actuator_move_target((data[1]<<8)+data[0]);

    current_pos= linear_actuator_get_position();
    return;

}


// TEST
void linear_actuator_reciprocating_test()
{

	// Reciprocating Test
	volatile int current_pos[3]={0,0,0},status=0;
	volatile int target_pos[2]={0,1500};
	volatile int target=0;
    volatile int status_cnt,move_cnt=0;

    // Initialize
	linear_actuator_calibrate_motor();				// <87 4> with wait for over 30msec

	//
	linear_actuator_set_speed(200,10,51);

	// Get the default position
	current_pos[0]= linear_actuator_get_position(); // <10>

	// Setting travel distance (NOT nesessary)
//	linear_actuator_set_travel_limits(0,3000);

	while(1){
		status_cnt=0;
		current_pos[1]= linear_actuator_get_position(); // <10>

		target = target_pos[move_cnt%2];
		linear_actuator_move_target(target); // <08>

		// Check the status of the motor
		while(linear_actuator_get_status())
			//linear_actuator_get_status() ==1
			//   (status_running(BIT2)==1) | status_motor_target(BIT18) == 0 | status_stalled(BIT23)==1)
		{
			status_cnt++;
		}

		status=linear_actuator_get_status();

		current_pos[2]= linear_actuator_get_position(); // <10>
		move_cnt++;
	}

	return ;

}





void linear_actuator_calibrate_motor(void){

	int i;
// Caribrate motor frequency
	const unsigned char TxDataInitialize[] =  {'<','8','7',' ','4','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};
	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataInitialize,(unsigned char *)TxDataInitialize);
	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM87);

	for(i=0;i<1000;i++)
	{
		_delay_cycles(1000);
	}


	return;
}


void linear_actuator_set_travel_limits(int foward_limit,int reverse_limig)
{
	unsigned char hex[HEX_DIGIT];
	unsigned char TxDataLinear[] =  {'<','4','6',' ','0','0','0','0','0','0','0','0',' ','0','0','0','0','0','0','0','0',' ','0','0','0','2','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};


	dec2hex(foward_limit,hex);
	TxDataLinear[11]=hex[0];
	TxDataLinear[10]=hex[1];
	TxDataLinear[ 9]=hex[2];
	TxDataLinear[ 8]=hex[3];

	dec2hex(reverse_limig,hex);
	TxDataLinear[20]=hex[0];
	TxDataLinear[19]=hex[1];
	TxDataLinear[18]=hex[2];
	TxDataLinear[17]=hex[3];

	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataLinear,(unsigned char *)TxDataLinear);
	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM46);

	return;
}


void linear_actuator_move_target(int pos){

	unsigned char TxDataLinear_pos[] =  {'<','0','8',' ','0','0','0','0','0','0','0','0','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};
	unsigned char hex[HEX_DIGIT];

	pos=(pos<0)?0:pos;

	dec2hex(pos,hex);
	TxDataLinear_pos[11]=hex[0];
	TxDataLinear_pos[10]=hex[1];
	TxDataLinear_pos[ 9]=hex[2];
	TxDataLinear_pos[ 8]=hex[3];

	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataLinear_pos,(unsigned char *)TxDataLinear_pos);

	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM8);

	return;
}






void linear_actuator_set_speed(int velocity,int acceleration,int cutoff)
{

	unsigned char TxDataLinear_pos[] =  {'<','4','0',' ','0','0','1','4','0','0',
													 ' ','0','0','0','0','0','A',
													 ' ','0','0','0','0','3','3',
													 ' ','0','0','0','1','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};
	unsigned char hex[HEX_DIGIT];

	dec2hex(velocity,hex);
	TxDataLinear_pos[ 9]=hex[0];
	TxDataLinear_pos[ 8]=hex[1];
	TxDataLinear_pos[ 7]=hex[2];
	TxDataLinear_pos[ 6]=hex[3];

	dec2hex(acceleration,hex);
	TxDataLinear_pos[16]=hex[0];
	TxDataLinear_pos[15]=hex[1];
	TxDataLinear_pos[14]=hex[2];
	TxDataLinear_pos[13]=hex[3];

	dec2hex(cutoff,hex);
	TxDataLinear_pos[23]=hex[0];
	TxDataLinear_pos[22]=hex[1];
	TxDataLinear_pos[21]=hex[2];
	TxDataLinear_pos[20]=hex[3];

	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataLinear_pos,(unsigned char *)TxDataLinear_pos);

	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM40);


	return;
}



int linear_actuator_get_position(){

	unsigned char TxDataLinear_pos[] =  {'<','1','0','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};
	volatile int result_dec;


	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataLinear_pos,(unsigned char *)TxDataLinear_pos);


	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM10);


	if(RxBuffer[0]=='<')
	{
		result_dec = hex2dec(RxBuffer[18])
									+(hex2dec(RxBuffer[17])<<4)
									+(hex2dec(RxBuffer[16])<<8)
									+(hex2dec(RxBuffer[15])<<12);
	}
	else{
		result_dec=-1;
	}




	return result_dec;
}



int linear_actuator_get_status(){

	unsigned char TxDataLinear_pos[] =  {'<','1','0','>',CARRIAGE_RETURN_LINEAR_ACTUATOR};
	volatile int status_upper,status_lower;

	volatile char status_running=0, status_motor_on_target=0,status_stalled=0;
	volatile char status_result=0;

	write_i2c(I2C_ADDR_LINEAR_ACTUATOR,sizeof TxDataLinear_pos,(unsigned char *)TxDataLinear_pos);


	read_i2c(I2C_ADDR_LINEAR_ACTUATOR,LENGTH_LINEAR_ACTUATOR_COM10);


	if(RxBuffer[0]=='<')
	{
		status_lower = hex2dec(RxBuffer[9])
									+(hex2dec(RxBuffer[8])<<4)
									+(hex2dec(RxBuffer[7])<<8)
									+(hex2dec(RxBuffer[6])<<12);

		status_upper=	 hex2dec(RxBuffer[5])
									+(hex2dec(RxBuffer[4])<<4);
	}
	else{
		status_upper=-1;
		status_lower=-1;
	}

	status_running=(status_lower & (0x0004)) >>2;
	status_motor_on_target=(status_upper & (0x0004)) >>2;
	status_stalled=(status_upper & (0x0080)) >>7;

	status_result= status_running;// | (status_motor_on_target==0)<<1 | status_stalled<<2;
	return status_result;
}

int hex2dec( unsigned char hex)
{
        int val=0;
        if(hex>='A'){
                val = (hex - 'A') + 10;
        }else{
                val = hex - '0';
        }
        return val;
}

unsigned int hex2dec_multiple(unsigned char *hex)
{
	unsigned int j, dec = 0;
	for (j = 0; j<HEX_DIGIT; j++)
	{
		dec += hex2dec(hex[j]) << (4 * j);
	}

	return dec;

}

