/*
 * i2c_read.c
 *
 *  Created on: Jun 15, 2015
 *      Author: makoto
 */

#include <msp430.h>
#include "bucket.h"
#include "i2c_com.h"
#include "servo.h"
#include "i2c_test.h"
#include "focal_stack.h"

#define __PULLUP_INSIDE

// Global
static const int scan_addr[MAX_NUM_SCAN_DOWNSTREAM]={I2C_ADDR_LENS,I2C_ADDR_STROBE,I2C_ADDR_MCU,I2C_ADDR_LINEAR_ACTUATOR};
static volatile unsigned char downstream_address[MAXMUM_CONNECTION];
static volatile unsigned char downstream_data[MAXMUM_CONNECTION];
static volatile unsigned char downstream_hierarchy[MAXMUM_CONNECTION];

static volatile unsigned char RXData;
static volatile int a_ack;
static volatile int tx_flag;
static volatile int slave_rx_flag;


// Master Read
static unsigned char RXByteCtr;
unsigned char RxBuffer[BUFFER_LENGTH];       // Allocate 128 byte of RAM
static unsigned char *PRxData;                     // Pointer to RX data
unsigned char CurrentBuffer[BUFFER_LENGTH];
static unsigned char mcu_buffer[BUFFER_LENGTH];

// Slave tx
unsigned char TxBuffer[BUFFER_LENGTH];
unsigned char *PTxData;                     // Pointer to TX data
extern volatile long read_cnt;
extern volatile long up_cnt;

unsigned char TXByteCtr;

unsigned char *SlavePRxData;                     // Pointer to RX data
unsigned char SlaveRXByteCtr;
volatile unsigned char SlaveRxBuffer[128];       // Allocate 128 byte of RAM

volatile unsigned char write_flag;
extern int servo_flag;

volatile  int degree_per_step;
extern  int current_degree;
extern  int target_degree;
volatile unsigned int servo_time;

const unsigned char TxData[] =              // Table of data to transmit
{
  0xA1,
  0xB2,
  0xC3,
  0xD4,
  0xE5,
  0xF6,
  0x07
};

struct I2C_WriteCommand I2C_write_command ;

unsigned char CommandStrobeSet[] =              // Table of data to transmit
{
  0x11,
  0x03,/*prescaler*/
  0xDC,/*duty*/
  0x05,/*prescaler*/
  0xe0,/*duty*/
  0xAA, /*Pattern*/
};



#define DOWNSTREAM_LED BIT1
#define UPSTREAM_LED BIT0

volatile char i2c_write_string[MAX_STR_LENGTH] = "";

int lens_pos=1000;
int bucket_main()
{
	volatile struct ComponentCode current_component_code;
	volatile unsigned char enable_address=0;
	int highest_adderss;

	volatile int ser_linear_actuator;


  // Initialize for current component code
  current_component_code.addr.height=0;
  current_component_code.addr.breadth=0;
#ifdef __BASE_COMPONENT
  current_component_code.type.major=TYPE_BASE;// TEMPORARY SETTING FOR EVA BOARD  Default: TYPE_BASE=0
#else
	#ifdef DEV_BOARD_F5510_1
	  current_component_code.type.major=((P2IN&(BIT5|BIT4))>>4) &0xF;//((P2IN&(BIT2|BIT3))>>2) & 0x3;// TEMPORARY SETTING FOR EVA BOARD  Default: TYPE_ACTUATOR=3
	#else
	  // After second dev board, the ID should be included in the source code.(No DIP switch)
	  current_component_code.type.major=CURRENT_COMPONENT_MAJOR;
	#endif
#endif



#ifdef __MSP430F5510__
  current_component_code.type.second=((P2IN&(BIT7|BIT6))>>6) &0xF;// TEMPORARY SETTING FOR DEVELOP BOARD
	#ifdef DEV_BOARD_F5510_1
	  current_component_code.type.second=((P2IN&(BIT5|BIT4))>>4) &0x3;// TEMPORARY SETTING FOR EVA BOARD
	#else
	  current_component_code.type.second=CURRENT_COMPONENT_SECOND;
	#endif
#endif



	  SlavePRxData = (unsigned char *)SlaveRxBuffer;    // Start of RX buffer
	  SlaveRXByteCtr = 0;                          // Clear RX byte count

	  // LED default setting
	#ifdef __BASE_COMPONENT
	  P1OUT=DOWNSTREAM_LED | UPSTREAM_LED;
	#else
	  // LED TURN OFF
	  P1OUT=0;
	#endif

	  // Initialize i2c
  init_i2c_UCB0();
  init_i2c_UCB1();


#ifndef __BASE_COMPONENT /* NON BASE COMPONENT */
  // To Upstream ( slave to master)
  __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0, enable interrupts
                                          // Remain in LPM0 until master
                                          // finishes RX
  memcpy(TxBuffer,CurrentBuffer,BUFFER_LENGTH);
  #endif



#ifndef __BASE_COMPONENT /* NON BASE COMPONENT */
// UpStream LED
#ifdef __MSP430F5510__
	#ifdef DEV_BOARD_F5510_1
		  P1OUT=0x10<<(up_cnt&0x3) | (P1OUT&0xF);
	#else
		  P1OUT=((P1OUT&UPSTREAM_LED)^UPSTREAM_LED)|(P1OUT&DOWNSTREAM_LED) ;
	#endif

	#else
		  P4OUT=BIT7;
	#endif


     // if this module receives data, P2OUT toggle
	if(slave_rx_flag)
	{


		I2C_write_command.addr   =SlaveRxBuffer[0];
		I2C_write_command.type   =SlaveRxBuffer[1];
		I2C_write_command.command=SlaveRxBuffer[2];
		I2C_write_command.data[0]=SlaveRxBuffer[3];
		I2C_write_command.data[1]=SlaveRxBuffer[4];
		I2C_write_command.data[2]=SlaveRxBuffer[5];
		I2C_write_command.data[3]=SlaveRxBuffer[6];
		I2C_write_command.reserved=SlaveRxBuffer[7];

		switch(I2C_write_command.command)
		{
		case I2C_COMMAND_SERVO_POSITION:
			target_degree=hex2dec_multiple(I2C_write_command.data);
			break;
		case I2C_COMMAND_SERVO_TIME:
			servo_time=hex2dec_multiple(I2C_write_command.data)*100;
			degree_per_step=get_servo_step_degree(current_degree,target_degree,servo_time);
			degree_per_step = ((target_degree-current_degree)>0)? degree_per_step:(-1)*degree_per_step;
			servo_flag=1;
			break;
		case I2C_COMMAND_SERVO_POSITION_DIRECT:
			set_pwm(hex2dec_multiple(I2C_write_command.data));
			current_degree=(int)I2C_write_command.data;
			break;

		case I2C_COMMAND_LINEAR_ACTUATOR_MOVE_TARGET:
			ser_linear_actuator= hex2dec_multiple(I2C_write_command.data);
			linear_actuator_move_target(ser_linear_actuator);
			break;

		case I2C_COMMAND_LINEAR_ACTUATOR_INITIALIZE:
			linear_actuator_calibrate_motor();
			break;
		case I2C_COMMAND_LINEAR_ACTUATOR_SET_VELOCITY:
			ser_linear_actuator= hex2dec_multiple(I2C_write_command.data);
			linear_actuator_set_speed(ser_linear_actuator,10,51);
			break;

		case I2C_COMMAND_FLASH_STROBE:
			strobe_set_pattern(hex2dec_multiple(I2C_write_command.data));
			break;
		case I2C_COMMAND_FLASH_STROBE_PRESCALE:
			CommandStrobeSet[1]=hex2dec_multiple(I2C_write_command.data);
			break;
		case I2C_COMMAND_FLASH_STROBE_DUTY:
			CommandStrobeSet[2]=hex2dec_multiple(I2C_write_command.data);
			break;

		default:
			break;

		}




//		servo_flag^=1;
//		control_servo(servo_flag);
		_NOP();

    	 slave_rx_flag=0;

	}
#endif

	//// if there is write transaction from up stream, just write it down stream.
     if(write_flag)
     {

    	 write_i2c(I2C_ADDR_MCU,WRITE_PACKET_LENGTH,(unsigned char *)i2c_write_string);

    	 write_flag=0;
    	 _NOP();
     }

     if(servo_flag)
     {

 		move_servo(degree_per_step);
 		  _NOP();
    	 _NOP();
     }



	// Master Rx //////////////////////////////////////////////////////////////////////

	  // Scan downstream
	  enable_address=scan_i2c(MAX_NUM_SCAN_DOWNSTREAM);


	  // DownStream LED
#ifdef __BASE_COMPONENT
#ifdef __MSP430F5510__
	#ifdef DEV_BOARD_F5510_1
		  P1OUT=enable_address?  (P1OUT&0xF) : (0xF0^(P1OUT&0xF0)) | (P1OUT&0xF);
	#else
		  P1OUT= enable_address? ((P1OUT&DOWNSTREAM_LED)^DOWNSTREAM_LED)|(P1OUT&UPSTREAM_LED) : (P1OUT&(DOWNSTREAM_LED|UPSTREAM_LED));
	#endif
#else
	  P4OUT=enable_address?  0:BIT7;
#endif
#else
	  P1OUT= enable_address? ((P1OUT&DOWNSTREAM_LED)^DOWNSTREAM_LED)|(P1OUT&UPSTREAM_LED) : 0;
#endif



	  // Read Loop start
	  int read_cnt;
	  for (read_cnt=DOWNSTREAM_SCAN_RATIO;read_cnt>0;read_cnt--)
	  {

	  // Initialize
	  int j;
	  for (j=0;j<BUFFER_LENGTH;j++) {		  CurrentBuffer[j]=0x0;	  }


	  volatile int length_current_data=0;

	  highest_adderss=  read_downstream(enable_address);


	// Generate Current Component Code
    current_component_code.start_code =I2C_COMMAND_START_OF_DATA;
    current_component_code.addr.height=enable_address?(highest_adderss+1):0;
    current_component_code.addr.breadth=0;
    current_component_code.setting.hi=0xBE;
    current_component_code.setting.lo=0xEF;
    current_component_code.connection=enable_address;

    // Added Current component code with the buffer
    CurrentBuffer[0]=length_current_data;// Length of data will be set later
    memcpy(CurrentBuffer+OFFSET_DATA_LENGTH_HEADER,(unsigned char*)&current_component_code,sizeof(current_component_code));
    length_current_data=OFFSET_DATA_LENGTH_HEADER+sizeof(current_component_code);


    // OFFSET_DOWNSTREAM_CODE
    int address_current_level=0;
    int i;
	volatile unsigned int length_mcu_data;
	volatile int flag_la=0;
    for(i=0;i<enable_address;i++)
    {
    	volatile struct ComponentCode downstream_component_code;
    	volatile struct ComponentCode attachment_component_code;

   		// Add Opening Parenthesis for Child
   	    CurrentBuffer[length_current_data++]=I2C_COMMAND_OPENING_BRANCH;


    	// Common setting for component code
    	downstream_component_code.start_code=I2C_COMMAND_START_OF_DATA;
    	downstream_component_code.addr.height=0x0;// Lowest is 0
		downstream_component_code.addr.breadth=address_current_level;
		downstream_component_code.type.major=TYPE_NO_DEFINE;
		downstream_component_code.type.second=TYPE_NO_DEFINE;//from data
 		downstream_component_code.setting.hi=0x0;
		downstream_component_code.setting.lo=0x0;
   		downstream_component_code.connection=0;

    	switch(downstream_address[i])
    	{
    	case I2C_ADDR_LENS:
    		downstream_component_code.addr.height=downstream_hierarchy[i];//depends on optical attachment
    		downstream_component_code.type.major=TYPE_LENS;
    		downstream_component_code.type.second=downstream_data[i]& 0xF;
       		downstream_component_code.connection=downstream_hierarchy[i]?1:0;// or 1
    		memcpy(CurrentBuffer+length_current_data,(unsigned char*)&downstream_component_code,sizeof(current_component_code));
    		length_current_data+=sizeof(current_component_code);

    		// Added data for optical attachment
    		if (downstream_component_code.addr.height==1)// Detected an optical attachment
    		{
    	   		// Add Opening Parenthesis for Child
    	   	    CurrentBuffer[length_current_data++]=I2C_COMMAND_OPENING_BRANCH;

    			attachment_component_code.start_code=I2C_COMMAND_START_OF_DATA;
    			attachment_component_code.addr.height=0;// Lowest is 0
    			attachment_component_code.addr.breadth=0;// One lens has only one attachment, therefore always 0
    			attachment_component_code.type.major=TYPE_OPTICAL_ATTACHMENT;
    			attachment_component_code.type.second=(0x70 & downstream_data[i])>>4;// From I2C expandar
    			attachment_component_code.setting.hi=0xAB;
    			attachment_component_code.setting.lo=0xCD;
    			attachment_component_code.connection=0;// One lens has only one attachment, therefore always 0
        		memcpy(CurrentBuffer+length_current_data,(unsigned char*)&attachment_component_code,sizeof(attachment_component_code));
        		length_current_data+=sizeof(attachment_component_code);
           		// Add Closing Parenthesis for Child
           	    CurrentBuffer[length_current_data++]=I2C_COMMAND_CLOSING_BRANCH;
    		}

    		break;
    	case I2C_ADDR_STROBE:
    		downstream_component_code.addr.height=0x0;
    		downstream_component_code.type.major=TYPE_STROBE;
    		downstream_component_code.type.second=1;//from downstream_data data
       		downstream_component_code.connection=0;
    		memcpy(CurrentBuffer+length_current_data,(unsigned char*)&downstream_component_code,sizeof(current_component_code));
    		length_current_data+=sizeof(current_component_code);

    		break;
    	case I2C_ADDR_LINEAR_ACTUATOR:
    		downstream_component_code.addr.height=downstream_hierarchy[i];//depends on optical attachment
    		downstream_component_code.type.major=TYPE_LENS;
    		downstream_component_code.type.second=LENS_FOCAL_STACK;
       		downstream_component_code.connection=0;
       		downstream_component_code.setting.hi=(unsigned char)(linear_actuator_get_position()/16);
			flag_la=linear_actuator_get_status();

			downstream_component_code.setting.lo=flag_la?0xFF:0;
    		memcpy(CurrentBuffer+length_current_data,(unsigned char*)&downstream_component_code,sizeof(current_component_code));
    		length_current_data+=sizeof(current_component_code);

    		break;
    	case I2C_ADDR_MCU:
    		// Overwrite horizontal address in mcu buffer
    		mcu_buffer[OFFSET_ADDRESS_HORIZONTAL]=address_current_level;
    		// Copy the downstream data
    		length_mcu_data=downstream_data[i];
    		memcpy(CurrentBuffer+length_current_data,mcu_buffer,sizeof(unsigned char)*(length_mcu_data));
    	    length_current_data+=length_mcu_data;
    		break;
    	default:
    		break;
    	}

   		// Add Closing Parenthesis for Child
   	    CurrentBuffer[length_current_data++]=I2C_COMMAND_CLOSING_BRANCH;

      	address_current_level++; // Increment horizontal address
    }

    // Set length of data
    length_current_data=length_current_data-OFFSET_DATA_LENGTH_HEADER;

    // Finished setting Current Bufffer
    CurrentBuffer[0]=length_current_data;


    // Master Rx //////////////////////////////////////////////////////////////////////








	 // Slave Tx  //////////////////////////////////////////////////////////////////////


      up_cnt++;


	    // SET BREAK POINT HERE
	    __no_operation();


	  }/* for read_cnt*/

	  return 0;

}


// USCI_B0_ISR
// Master Rx
// Wait mode
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_B0_VECTOR))) USCI_B0_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch(__even_in_range(UCB0IV,12))
  {
  case  0: break;                           // Vector  0: No interrupts
  case  2: break;                           // Vector  2: ALIFG
  case  4:
		a_ack=I2C_DETECT_NACK;// For detect
//	    __bic_SR_register_on_exit(LPM0_bits);   // Exit active CPU
	  break;                           // Vector  4: NACKIFG

  case  6: break;                           // Vector  6: STTIFG
  case  8:                          // Vector  8: STPIFG
  __bic_SR_register_on_exit(LPM0_bits);   // Exit active CPU
  break;
  case 10:                                  // Vector 10: RXIFG
	  // Read Multi Byte
	    RXByteCtr--;                            // Decrement RX byte counter
		a_ack=I2C_DETECT_ACK;// For detect
	    if (RXByteCtr)
	    {
	      *PRxData++ = UCB0RXBUF;               // Move RX data to address PRxData
	      if (RXByteCtr == 1)                   // Only one byte left?
	        UCB0CTL1 |= UCTXSTP;                // Generate I2C stop condition
	    }
	    else
	    {
	      *PRxData = UCB0RXBUF;                 // Move final RX data to PRxData
	    }


    break;
  case 12:
  if (TXByteCtr)                          // Check TX byte counter
  {
    UCB0TXBUF = *PTxData++;               // Load TX buffer
    TXByteCtr--;                          // Decrement TX byte counter
  }
  else
  {
    UCB0CTL1 |= UCTXSTP;                  // I2C stop condition
    UCB0IFG &= ~UCTXIFG;                  // Clear USCI_B0 TX int flag
    __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
  }
  break;                           // Vector 12: TXIFG
  default: break;
  }
}


// USCI_B1_VECTOR
// Port 4 Upstream
//--Slave tx -- IRQ
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = USCI_B1_VECTOR
__interrupt void USCI_B1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_B1_VECTOR))) USCI_B1_ISR (void)
#else
#error Compiler not supported!
#endif

{
  switch(__even_in_range(UCB1IV,12))
  {
  case  0: break;                           // Vector  0: No interrupts
  case  2: break;                           // Vector  2: ALIFG
  case  4: break;                           // Vector  4: NACKIFG
  case  6:                                  // Vector  6: STTIFG
	  PTxData = (unsigned char *)TxBuffer;      // Start of TX buffer
	  UCB1IFG &= ~UCSTTIFG;                   // Clear start condition int flag
    break;
  case  8:                                  // Vector  8: STPIFG
    UCB1IFG &= ~UCSTPIFG;                   // Clear stop condition int flag
    __bic_SR_register_on_exit(LPM0_bits);   // Exit LPM0 if data was transmitted
    break;
  case 10:                            // Vector 10: RXIFG
	  *SlavePRxData++ = UCB1RXBUF;                 // Get RX'd byte into buffer
  	  SlaveRXByteCtr++;
  	  slave_rx_flag=1;
  break;
  case 12:                                  // Vector 12: TXIFG
    UCB1TXBUF = *PTxData++;                 // Transmit data at address PTxData
    break;
  default: break;
  }
}





/*UCB0(PORT3)  Down Stream Master Rx*/
void init_i2c_UCB0()
{
	  //UCB0(PORT3) Master Rx
	  UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	  UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
	  UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
	  UCB0BR0 = I2C_CLK;                             // fSCL = SMCLK/12 = ~100kHz
	  UCB0BR1 = 0;
	  UCB0I2CSA = I2C_ADDR_MCU;                         // Slave Address is 048h
//      UCB0I2CSA = 0x48;
	  UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation

	  // ** CAUTION ** Enable must be after resume operation //
	  // Interrupt enable for Master Tx
	  UCB0IE |= UCRXIE;             // RxData
	  UCB0IE |= UCNACKIE;           // NACK
	  UCB0IE |= UCSTPIE;			// STOP
	  UCB0IE |= UCTXIE;                         // Enable TX interrupt

	  return;

}




/*UCB1 Up stream Slave Tx*/
void init_i2c_UCB1()
{

	  //UCB1(PORT4) Slave Tx
	  UCB1CTL1 |= UCSWRST;                      // Enable SW reset
	  UCB1CTL0 = UCMODE_3 + UCSYNC;             // I2C Slave, synchronous mode
	  UCB1I2COA = I2C_ADDR_MCU;                         // Own Address is 024h
	  //UCB1I2COA = 0x48;                         // Own Address is 048h

	  UCB1CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation

	  // Interrupt enable for Slave Rx
	  // ** CAUTION ** Enable must be after resume operation //
	  UCB1IE |= UCTXIE + UCSTPIE + UCSTTIE;     // Enable STT and STP interrupt
	                                            // Enable TX interrupt
	  UCB1IE |= UCRXIE;             // RxData

	  return;
}





int scan_i2c(unsigned int addr_cnt)
{
	// Initialize before scan
	unsigned int enable_addr=0;

	unsigned int i;

	// Initilize the array for data
	for (i=0;i<MAXMUM_CONNECTION;i++){
	  downstream_address[i]=0;
	  downstream_data[i]=0;
	}

	// Scan
	for(i=0;i<addr_cnt;i++)
	{
		if(read_i2c(scan_addr[i],1))
		{
			downstream_address[enable_addr]=scan_addr[i];
			downstream_data[enable_addr]=RxBuffer[0];
			enable_addr++;
		}
	}

	return enable_addr;
}



// Read for wait mode(Not IRQ)
int read_i2c(unsigned int addr, unsigned int length_data)
{
	int retry,i,tm;
	retry=I2C_RETRY_MAX;

	// Setting Data Length
	RXByteCtr=length_data;

	// Setting read address


	UCB0I2CSA=addr;
	a_ack=I2C_DETECT_NACK;
    PRxData = (unsigned char *)RxBuffer;    // Start of RX buffer

    while((a_ack==I2C_DETECT_NACK) && (retry--)>0)
    {

		// Start
		tm=I2C_IRQ_TIMEOUT;

		// Ensure stop condition got sent
		while ((UCB0CTL1 & UCTXSTP)&&(tm>0)){--tm;};

		UCB0CTL1 |= UCTXSTT;                    // I2C start condition +read Not |)
	    __bis_SR_register( GIE);     // Enter LPM0, enable interrupts

		 // Ensure Start condition sent for 1 byte
		if (length_data==1)
		{
			tm=I2C_IRQ_TIMEOUT;
			while((UCB0CTL1 & UCTXSTT)&&(tm>0)){--tm;};
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
		}
		// wait
		for (i=length_data;i>0;i--)
		{
			__delay_cycles(I2C_IRQ_WAIT_TIME);
		}
		__bis_SR_register( 0);

		__no_operation();

    }

	return a_ack;


}


// detect the highest address of hierarchy and get MCU data again
unsigned int read_downstream(int enable_address)
{
	unsigned int i,highest_adderss=0;
	volatile unsigned int length_mcu_data=0;
	volatile unsigned int check_data=0;
	 for(i=0;i<enable_address;i++)
	 {
		//  If MCU is detected, then read multi bytes data
		switch(downstream_address[i])//I2C_ADDR_MCU_LENGH_DATA)
		{
		case I2C_ADDR_MCU:
			length_mcu_data=downstream_data[i]; // except the data length

			// Read *LONG* MCU data //////////////////////
			read_i2c(I2C_ADDR_MCU,length_mcu_data+OFFSET_DATA_LENGTH_HEADER); /*length_data*/
			// detect hierarchy of downstream module
			downstream_hierarchy[i]=RxBuffer[OFFSET_DATA_LENGTH_HEADER+OFFSET_ADDRESS_HIERARCHY];
			memcpy(mcu_buffer,RxBuffer+OFFSET_DATA_LENGTH_HEADER,sizeof(unsigned char)*length_mcu_data);

			break;
		case I2C_ADDR_LENS:
			// if Bit[7:4]==F then no optical attachment, hierarchy of Lens is 0
			downstream_hierarchy[i]=((0xF0 & downstream_data[i])==0xF0)?0:1;
			break;
		default:
			downstream_hierarchy[i]=0;
			break;
		}

		//  detected highest hierarchy
		if (highest_adderss<downstream_hierarchy[i])
			highest_adderss=downstream_hierarchy[i];

		 __no_operation();                       // Set breakpoint >>here<< and
	 }

	return highest_adderss;

}



int write_i2c(unsigned int addr, unsigned int length_data, unsigned char *TxDataBuf)
{
	int j;
	 UCB0I2CSA =addr;

	__delay_cycles(10);

	PTxData = TxDataBuf;      // TX array start address
											// Place breakpoint here to see each
											// transmit operation.
	TXByteCtr = length_data;              // Load TX byte counter

	UCB0CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition

#ifndef __IRQ_I2C
	__bis_SR_register(  GIE);     // Enable interrupts
	for(j=0;j<WRITE_PACKET_LENGTH;j++)
	{
	_delay_cycles(1000);
	}
#else
	__bis_SR_register( LPM0_bits + GIE);     // Enter LPM0 w/ interrupts
	__no_operation();                       // Remain in LPM0 until all data
											// is  RX'd
#endif

	while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent


  	init_i2c_UCB0();

	return 0;
}

//#define __IRQ_I2C_RX

int i2c_slave_rx()
{
	slave_rx_flag=0;
	  SlavePRxData = (unsigned char *)SlaveRxBuffer;    // Start of RX buffer
	  SlaveRXByteCtr = 0;                          // Clear RX byte count

#ifndef __IRQ_I2C_RX
      __bis_SR_register(  GIE);     // Enable interrupts
      __delay_cycles(1000);
#else
  __bis_SR_register( LPM0_bits + GIE);     // Enter LPM0 w/ interrupts
  __no_operation();                       // Remain in LPM0 until all data
  										// is  RX'd
#endif
  __no_operation();                       // Set breakpoint >>here<< and read


	return 0;

}



void strobe_set_pattern(int pattern)
{

	CommandStrobeSet[5]=pattern;

	    // Initialize i2c
	init_i2c_UCB0();

	write_i2c(I2C_ADDR_STROBE,sizeof CommandStrobeSet,(unsigned char *)CommandStrobeSet);

	return;
}

