// Servo Demo
//               MSP430F5510
//            -----------------
//        /|\|              XIN|-
//         | |                 | 32kHz
//         --|RST          XOUT|-
//           |                 |
//           |             P2.0|-->TA1.0 (PWM)
// Just turn around
// Makoto Odamaki
//*****************************************************************************

#include <msp430.h>
#include "bucket.h"
#include "servo.h"


volatile int servo_flag;
volatile  int current_degree;
volatile  int target_degree;



void servo_initialize()
{

	volatile int pwm_period,duty;

	WDTCTL = WDTPW + WDTHOLD;  // Stop WDT

	// Calculate the duration
	pwm_period=(int)( PWM_PERIOD/XT1_CYCLE) ;
	duty=(int) ( NUETRAL_DUTY/XT1_CYCLE) ;

	/// SETTING GSetting Clock
	  //  ACLK= Peripheral
	  //  MCLK= Main CPU		SELM__DCOCLKDIV
	  //  SMCLK=Sub Main Clock  XT1CLK
//	UCSCTL4 = SELS__XT1CLK;// <-----------------------
	UCSCTL4 |= SELA__XT2CLK;
	// SETTING TIMER
  	  TA1CCR0 = 10593;//pwm_period;             // PWM Period
	  TA1CCR1 = 1589;                // CCR1 PWM duty cycle

	  TA1CCTL1 = OUTMOD_7;          // CCR1 reset/set
	  TA1CTL = TASSEL_1 + MC_1;   // ACLK, up mode
	  // TASSEL
	  //  01b ACLK= Peripheral
	  //  10b SMCLK=Sub Main Clock

	  current_degree=30;
	  set_pwm(current_degree);
}


void set_pwm(volatile int degree)
{
	volatile float duty = ((HS5035HD_MAX_DUTY-HS5035HD_NEUTRAL_DUTY)*degree)/HS5035HD_MAX_DEGREE+HS5035HD_ZERO_DUTY;
	duty/=XT2_CYCLE;

	volatile int duty_int= (int)(duty+0.5);
	TA1CCR1 = duty_int;
	return;
}


int get_servo_step_degree(int current_position,int target_position,int time)
// time-- msec
{
// DURATION_COMPONENT_MSEC
    volatile float step_degree;
    volatile int diff_positon;

    diff_positon=target_position-current_position;
    diff_positon=(diff_positon<0)?-diff_positon:diff_positon;

    step_degree=diff_positon/(time/DURATION_COMPONENT_MSEC);

    step_degree = ((step_degree<1) & (step_degree>=0))?1:step_degree;
    step_degree = ((step_degree<0) & (step_degree>-1))?-1:step_degree;

    return (int)step_degree;
}


void move_servo(int degree_per_step){
	set_pwm(current_degree);
	current_degree+=degree_per_step;
	  if(degree_per_step>0)
	  {
		  if(current_degree>target_degree)
		  {
			  servo_flag=0;
		  }

	  } else
	  {
		  if(current_degree<target_degree)
		  {
			  servo_flag=0;
		  }
	  }
}

void servo_test()
{
	int i,j;

	for(i=30;i<150;i++){
		set_pwm(i);

		for(j=0;j<50;j++){
			  __delay_cycles(SERVO_DELAY);
		}

	}


	for(i=150;i>30;i--){
		set_pwm(i);
		for(j=0;j<50;j++){
			  __delay_cycles(SERVO_DELAY);
		}	}

	return;
}



